/**************************************************************************
 *
 *	project				 : VisiScript
 *
 *	copyright            : (C) 2002 by Michael Neuroth
 *
 * ------------------------------------------------------------------------
 *
 *  $Source$
 *
 *  $Revision$
 *
 *	$Log$
 *
 ***************************************************************************/
/*********************************************************************************
 *																				 *
 * This file is part of the VisiScript package (a frontend for script languages) *
 *																				 *
 * Copyright (C) 2002 by Michael Neuroth.										 *
 *                                                                               *
 * This program is free software; you can redistribute it and/or modify			 *
 * it under the terms of the GNU General Public License as published by    		 *
 * the Free Software Foundation; either version 2 of the License, or             *
 * (at your option) any later version.                                           *
 *                                                                               *
 ********************************************************************************/

#ifndef VISISCRIPT_H
#define VISISCRIPT_H

#include <qlist.h>

#include <qmainwindow.h>

// Problems with save-as functionality with opie file-dialog
//#define _with_opie_filedialog

class FileSelector;
class QToolBar;
class QMenuBar;
class QWidgetStack;
class QVBox;
class QMultiLineEdit;
class QTextView;
class QPopupMenu;
class QAction;
class QTabWidget;
class QLabel;
class QWidget;
class QProcess;
class QextScintillaAPIs;

class MyQMultiLineEdit;		// to invoke the method hasMarkedText()
class MyQCodeEdit;
class FileDialogS;
class QextScintillaLexer;

#include "codeedit.h"

// ************************************************************************

class VisiScript : public QMainWindow
{
    Q_OBJECT

public:
    VisiScript( QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
    ~VisiScript();

#ifndef ZAURUS
	virtual void dragEnterEvent( QDragEnterEvent * pEvent );
	virtual void dropEvent( QDropEvent * pEvent );
#endif

	virtual bool DoImportFile( const QString & sFileName );

public slots:
    void sltFileNew();
    virtual void sltFileOpen();
    virtual void sltFileSave();
    virtual void sltFileSaveAs();
    void sltFileSaveOutputAs();
    void sltFileImport();
    void sltFileExport();
    void sltEditUndo();
    void sltEditRedo();
    void sltEditCut();
    void sltEditCopy();
    void sltEditPaste();
    void sltEditFindText();
    void sltEditFindNext();
    void sltEditReplaceText();
    void sltClearOutput();
	void sltCommentSelection();
	void sltTabWithSelectedTextPressed();
	void sltShiftTabWithSelectedTextPressed();
	void sltUnCommentSelection();
	void sltAddLineToOutput( const QString & sLine );
    void sltClearScript();
	void sltStartInteractiveModus( bool bEnable );
	void sltStopInteractiveModus();
    void sltRun();
    void sltSelectFont();
	//void sltSelectHelvetica();
	//void sltSelectFixed();
	//void sltSelectUnifont();
	//void sltSelectSmall();
	//void sltSelectMedium();
	//void sltSelectLarge();
	void sltZoomIn();
	void sltZoomOut();
	void sltMaximizeOutput();
	void sltMaximizeScript();
	void sltToggleWordWrap();
	void sltToggleOverwrite();
	void sltCodeFolding();
    void sltShowAbout();
	void sltUpdateLastFilesMenu();
	void sltLastFilesMenuActivated( int iIndex );

	void sltUpdateChilds();

	virtual void show();

	void sltPositionChanged( int iLine, int iPos );
	void sltReadFromStdout();
	void sltReadFromStderr();
	void sltProcessExited();
	void sltWriteToStdin( const QString & sText );

protected slots:
    void sltNewFile();
    void sltConfiguration();
    void sltShowScript();
    void sltFileAboutToShow();
    void sltEditAboutToShow();
    void sltViewAboutToShow();
    void sltLanguagesAboutToShow();
    void sltLanguageActivated( int iId );
#ifndef _with_opie_filedialog
    void sltDoImport();
    void sltDoExport();
#endif
	void sltCloseActDocument();
	virtual void sltClearActDocument();

protected:
    virtual void closeEvent( QCloseEvent * pCloseEvent );

    void updateCaption( const QString & sName=QString::null );
    virtual void updateFileSelector();
    void loadInit();
    void saveInit();
    bool CheckActData();
	bool CheckAllData();
    void UpdateFont( bool bForAllDocuments );
	void UpdateFont( MyQCodeEdit * pScript, MyQMultiLineEdit * pOutput );
    void UpdateFont( MyQCodeEdit * pScript, MyQMultiLineEdit * pOutput, const QFont & aFont );
    void UpdateFlags();
    void UpdateLanguage( bool bForAllDocuments );
    void UpdateLanguage( MyQCodeEdit * pScript );
    void UpdateLanguageMenu();
    void UpdateScriptOutputRatio();
	void InsertInSelection( bool bDoInsert, const QString & sToken );
    int GetHeightForOutput( int iPercent );
	void PushLastFileName( const QString & sName );
	bool ExistsLastFileName( const QString & sName );
	bool DoExportFile( const QString & sFileName );
	bool DoExportFile( MyQCodeEdit * pScript );
	void SetOutputSize( int iPercentSize );
	MyQCodeEdit * GetActScript() const;
	MyQMultiLineEdit * GetActOutput() const;
	MyQCodeEdit * GetScriptFor( int iIndex ) const;
	MyQMultiLineEdit * GetOutputFor( int iIndex ) const;
	QWidget * GetActDocumentTabPage() const;
	int GetDocumentCount() const;
	void AddDocument( int iDocCount, const QString & sFileName = tr( "empty" ) );
	bool doFindFirst( int iLine = -1, int iIndex = -1 );
	void SetAutoCompletionSource( QextScintilla::AutoCompletionSource aSrc, int iThresh, QextScintillaAPIs * pAPIs );
	QString GetStringForTab() const;

	template <class T1>
	void ForAllDocumentScripts( T1 pFcn );
	template <class T1, class T2>
	void ForAllDocumentScripts( T1 aValue, T2 pFcn );
	template <class T1, class T2, class T3>
	void ForAllDocumentScripts( T1 aValue1, T2 aValue2, T3 pFcn );

	template <class T1>
	void ForAllDocumentOutput( T1 pFcn );
	template <class T1, class T2>
	void ForAllDocumentOutput( T1 aValue, T2 pFcn );

	// ** data **
#ifndef _with_opie_filedialog
	FileDialogS *		m_pExportFileDlg;
#endif
#ifndef ZAURUS
	QLabel *			m_pStatusBarMsg;
#endif
	QProcess *			m_pProcess;

	FileSelector *		m_pFileSelector;
	QToolBar *			m_pMenu;
	QToolBar *			m_pButtonBar;
	QMenuBar *			m_pMenuBar;			// QPEMenuBar
	QWidgetStack *		m_pStack;
	QTabWidget *		m_pTabWidget;

    QPopupMenu * 		pFile;
    QPopupMenu * 		pEdit;
	QPopupMenu * 		pView;
	QPopupMenu *		pLanguages;
    QPopupMenu * 		pLastOpendFiles;

	//QAction *			m_pFixedAction;
	//QAction *			m_pHelveticaAction;
	//QAction *			m_pUnifontAction;
	//QAction *			m_pSmallAction;
	//QAction *			m_pMediumAction;
	//QAction *			m_pLargeAction;
	QAction *			m_pMaximizeScript;
	QAction *			m_pMaximizeOutput;
	QAction *			m_pToggleWordWrap;
	QAction *			m_pToggleOverwrite;
	QAction *			m_pCodeFolding;
	QAction *			m_pCopyAction;
	QAction *			m_pCutAction;
	QAction *			m_pPasteAction;
	QAction *			m_pCommentAction;
	QAction *			m_pUnCommentAction;

	QString				m_sScriptCmd;
	QString				m_sScriptExtention;

	QString				m_sFontName;
	QString				m_sFontStyle;
	int					m_iFontSize;

	bool				m_bSaveOutput;

	int					m_iWordWrap;
	int					m_iOverwrite;		// temp
	int					m_iSpacesForTabs;
	int					m_iShowWhitespaces;
	int					m_iShowLineNumbers;
	int					m_iAutoIndent;
	int					m_iCodeFolding;
	int					m_iTabSpace;
	int					m_iActLanguageID;
	int					m_iEolMode;
	QString				m_sLastDirectory;
	QString				m_sExeclCmd;
	int					m_iExecuteMode;		// 1 == popen, 2 == fork/execl, 3 == QProcess
	int					m_iShowStartupFileDialog;
	int					m_iOutputRatio;		// for the ratio of Script and output
	QString				m_sFindText;		// temps for find/replace dialog
	QString				m_sReplaceText;
	bool				m_bIsReplace;
	bool				m_bRegularExpr;
	bool				m_bCaseSensitive;
	bool				m_bOnlyWholeWords;
	bool				m_bSearchBackward;
	bool				m_bReplaceAll;
	bool				m_bInteractiveModus;
	bool				m_bBraceMatching;
	bool				m_bAutoClearOutput;
	int					m_iActLine;
	int					m_iActPos;
	int					m_iAutoCompletionThresh;
	QString				m_sAutoCompletionFileName;
	QextScintillaAPIs * m_pAutoCompletionAPI;

	QList<QString>		m_aLastFilesList;
	QList<QString>		m_aLastSearchStringList;
	QList<QString>		m_aLastReplaceStringList;
};

const int EXECUTE_MODE_POPEN = 1;
const int EXECUTE_MODE_EXECL = 2;
const int EXECUTE_MODE_QPROCESS = 3;

#endif // VISISCRIPT
