
#include "zvisiscript.h"
#include "SaveAsDlg.h"

#include <qaction.h>
#include <qlineedit.h>
#include <qcombobox.h>
#include <qwidgetstack.h>

#include <qpe/qpetoolbar.h>
#include <qpe/fileselector.h>

static void copyDocLnk( const DocLnk & source, DocLnk & target )
{
	target.setName(source.name());
	target.setFile(source.file());
	target.setLinkFile(source.linkFile());
	target.setComment(source.comment());
	target.setType(source.type());
	target.setCategories(source.categories());
}

static int g_iDocCount = 0;

ZVisiScript::ZVisiScript( QWidget* parent, const char* name, WFlags fl )
: VisiScript( parent, name, fl )
{
	m_pGotoBegin = new QAction( tr( "Begin" ), QString::null, CTRL+Key_B, this, 0, TRUE );
    connect( m_pGotoBegin, SIGNAL( activated() ), this, SLOT( sltGotoBegin() ) );
	m_pGotoEnd = new QAction( tr( "End" ), QString::null, CTRL+Key_E, this, 0, TRUE );
    connect( m_pGotoEnd, SIGNAL( activated() ), this, SLOT( sltGotoEnd() ) );

	AddDocument( g_iDocCount++ );

	updateFileSelector();

	if( m_iShowStartupFileDialog )
	{
	    sltFileOpen();
	}
	else
	{
		sltShowScript();
		updateCaption();
	}
}

void ZVisiScript::sltNewFileDoc( const DocLnk & f )
{
    DocLnk nf = f;
    nf.setType( /*"text/plain"*/m_sScriptExtention );	// Bugfix 30.8.2003
    nf.setName( tr( "empty" ) );

	copyDocLnk( nf, m_aDoc );

	sltFileNew();

    //updateCaption( m_aDoc.name() );
}

void ZVisiScript::sltOpenFileDoc( const DocLnk & f )
{
	// ACHTUNG: man MUSS eine deep-copy durchfuehren !!!
	//error:	m_aDoc = f;
	copyDocLnk( f, m_aDoc );

    //sltClearScript();		// not needed for multiple documents !

    FileManager fm;
    QString txt;
    if ( !fm.loadFile( f, txt ) ) {
	// ####### could be a new file
	//qDebug( "Cannot open file" );
	//return;
    }

    sltShowScript();

	AddDocument( -1, m_aDoc.name() );

    GetActScript()->setText( txt );
    GetActScript()->setEdited(FALSE);

	PushLastFileName( m_aDoc.file() );

    //updateCaption( m_aDoc.name() );
}

void ZVisiScript::sltGotoBegin()
{
	GetActScript()->setCursorPosition( 0, 0 );
}

void ZVisiScript::sltGotoEnd()
{
	GetActScript()->setCursorPosition( GetActScript()->lines(),
						GetActScript()->lineLength( GetActScript()->lines() ) );
}

void ZVisiScript::sltFileSaveAs()
{
	SaveAsDlg * pDlg = new SaveAsDlg( this, "saveasdlg", /*bModal*/TRUE );

	pDlg->m_pFileName->setText( tr( "empty" ) );

	pDlg->adjustSize();

	int iRet = pDlg->exec();

	if( iRet == 1 )
	{
		// ok, pressed
		QString sFileName = pDlg->m_pFileName->text();

	    FileManager fm;

		QString s = GetActScript()->text();
		QString sExt = pDlg->m_pType->currentText();

		// create a new Doc-Link for the copy of the actual file
		DocLnk aDoc;
		aDoc.setType( /*"text/plain"*/sExt );
		aDoc.setName( sFileName );
	    if( fm.saveFile( aDoc, s ) )
		{
			// schreiben war ok
			copyDocLnk( aDoc, m_aDoc );
			//updateCaption( m_aDoc.name() );

			GetActScript()->setFileName( m_aDoc.file() );
			GetActScript()->setEdited( false );

			PushLastFileName( m_aDoc.file() );
		}
		else
		{
			// ggf. todo --> fehlermeldung ausgeben !
		}
	}

	delete pDlg;
}

bool ZVisiScript::DoImportFile( const QString & sFileName )
{
	if( VisiScript::DoImportFile( sFileName ) )
	{
		// update/clear doc-link !
		DocLnk aDoc;
		//aDoc.setType( "text/plain" );
		//aDoc.setName( *pFileName );
		copyDocLnk( aDoc, m_aDoc );

		return true;
	}
	return false;
}

void ZVisiScript::sltFileOpen()
{
    //CheckData();		// not needed for multiple documents !

    m_pMenu->hide();
    m_pButtonBar->hide();
    clearWState (WState_Reserved1 );
    updateFileSelector();
    m_pStack->raiseWidget( m_pFileSelector );
    m_pFileSelector->reread();
    updateCaption();
}

void ZVisiScript::sltFileSave()
{
    FileManager fm;

	QString s = GetActScript()->text();
    if( fm.saveFile( m_aDoc, s ) )
	{
		GetActScript()->setEdited( false );
    }
	else
	{
		// ggf. todo --> fehlermeldung ausgeben !
	}
}

void ZVisiScript::updateFileSelector()
{
    delete m_pFileSelector;

    m_pFileSelector = new FileSelector( m_sScriptExtention, m_pStack, "fileselector" , TRUE, FALSE );
    connect( m_pFileSelector, SIGNAL( closeMe() ), this, SLOT( sltShowScript() ) );
    connect( m_pFileSelector, SIGNAL( newSelected( const DocLnk &) ), this, SLOT( sltNewFileDoc( const DocLnk & ) ) );
    connect( m_pFileSelector, SIGNAL( fileSelected( const DocLnk &) ), this, SLOT( sltOpenFileDoc( const DocLnk & ) ) );
}

